@@ -51,6 +51,8 @@ class PowerProfile():
         self.curve = pd.DataFrame(data)
         self.curve.sort_values(by=self.datetime_field, inplace=True)
         self.curve.reset_index(inplace=True, drop=True)
+        # Ensure timestamp field is localized
+        self.curve[self.datetime_field] = self.curve.apply(lambda row: self.ensure_localized_dt(row), axis=1)
 
         if start:
             self.start = start
@@ -72,6 +74,13 @@ class PowerProfile():
                     auto_data_fields.append(field)
             self.data_fields = auto_data_fields
 
+    def ensure_localized_dt(self, row):
+        dt = row[self.datetime_field]
+        if dt.tzinfo is None:
+            return TIMEZONE.localize(dt)
+        else:
+            return dt
+
     def dump(self):
 
         data = self.curve.to_dict(orient='records')
@@ -96,7 +105,8 @@ class PowerProfile():
             ids = set(self.curve[self.datetime_field])
             dt = start
             df_hours = set([TIMEZONE.normalize(dt + timedelta(hours=x)) for x in range(0, int(hours))])
-            first_not_found = list(df_hours - ids)[0]
+            not_found = sorted(list(df_hours - ids))
+            first_not_found = not_found[0]
             return False, first_not_found
         return True, None
 
@@ -120,7 +130,7 @@ class PowerProfile():
         uniques = len(self.curve[self.datetime_field].unique())
         if uniques != self.hours:
             ids = self.curve[self.datetime_field]
-            first_occurrence = self.curve[ids.isin(ids[ids.duplicated()])][self.datetime_field][0]
+            first_occurrence = self.curve[ids.isin(ids[ids.duplicated()])][self.datetime_field].min()
             return True, first_occurrence
         return False, None
 
@@ -168,7 +178,7 @@ class PowerProfile():
             powpro.end = res.iloc[-1][self.datetime_field]
             return powpro
         elif isinstance(item, datetime):
-            if not datetime.tzinfo:
+            if not item.tzinfo:
                 raise TypeError('Datetime must be a localized datetime')
 
             res = self.curve.loc[self.curve[self.datetime_field] == item]
@@ -425,6 +435,35 @@ class PowerProfile():
         )
         return csvfile.getvalue()
 
+    def get_complete_daily_subcurve(self):
+        """
+        Returns partial curve from first hour to last complete day without gaps nor duplicities
+        :return: dataframe
+        """
+        first_gap = None
+        iscomplete, gap = self.is_complete()
+        if gap is not None:
+            first_gap = gap
+
+        hasduplicates, gap = self.has_duplicates()
+        if gap is not None:
+            first_gap = min(first_gap, gap)
+
+        if first_gap is None:
+            return self
+        else:
+            last_hour = TIMEZONE.normalize(first_gap - timedelta(hours=1))
+            if last_hour.hour > 0:
+                last_hour = last_hour.replace(hour=0)
+            if last_hour >= self.start:
+                data = self.curve[self.curve[self.datetime_field] <= last_hour]
+                data = data.to_dict('records')
+                res = PowerProfile()
+                res.load(data, datetime_field=self.datetime_field)
+            else:
+                res = PowerProfile()
+            return res
+
 
 class PowerProfileQh(PowerProfile):
 
